@@ -55,7 +55,7 @@ extern void dump_thread(struct pt_regs *, struct user *);
 
 /*
  * Here are the actual binaries that will be accepted:
- * add more with "register_binfmt()"..
+ * add more with "register_binfmt()" if using modules...
  */
 extern struct linux_binfmt elf_format;
 
@@ -69,6 +69,7 @@ static struct linux_binfmt aout_format = {
 
 static struct linux_binfmt *formats = &aout_format;
 
+#ifdef CONFIG_MODULES
 int register_binfmt(struct linux_binfmt * fmt)
 {
 	struct linux_binfmt ** tmp = &formats;
@@ -100,6 +101,7 @@ int unregister_binfmt(struct linux_binfmt * fmt)
 	}
 	return -EINVAL;
 }
+#endif	/* CONFIG_MODULES */
 
 int open_inode(struct inode * inode, int mode)
 {
@@ -328,6 +330,7 @@ unsigned long * create_tables(char * p, struct linux_binprm * bprm, int ibcs)
 		mpnt->vm_inode = NULL;
 		mpnt->vm_pte = 0;
 		insert_vm_struct(current, mpnt);
+		current->mm->total_vm = (mpnt->vm_end - mpnt->vm_start) >> PAGE_SHIFT;
 	}
 	sp = (unsigned long *) ((-(unsigned long)sizeof(char *)) & (unsigned long) p);
 #ifdef __alpha__
@@ -541,9 +544,12 @@ static void exec_mmap(void)
 			return;
 		}
 		*mm = *current->mm;
+		mm->def_flags = 0;	/* should future lockings be kept? */
 		mm->count = 1;
 		mm->mmap = NULL;
 		mm->mmap_avl = NULL;
+		mm->total_vm = 0;
+		mm->rss = 0;
 		current->mm->count--;
 		current->mm = mm;
 		new_page_tables(current);
@@ -662,7 +668,7 @@ restart_interp:
 		goto exec_error2;
 	}
 	/* better not execute files which are being written to */
-	if (bprm.inode->i_wcount > 0) {
+	if (bprm.inode->i_writecount > 0) {
 		retval = -ETXTBSY;
 		goto exec_error2;
 	}
@@ -744,8 +750,13 @@ restart_interp:
 		goto restart_interp;
 	}
 #ifdef __alpha__
-/* handle /sbin/loader.. */
-	if (!loader && (((struct exec *) bprm.buf)->fh.f_flags & 0x3000)) {
+	/* handle /sbin/loader.. */
+	{
+	    struct exec * eh = (struct exec *) bprm.buf;
+
+	    if (!loader && eh->fh.f_magic == 0x183 &&
+		(eh->fh.f_flags & 0x3000) == 0x3000)
+	    {
 		char * dynloader[] = { "/sbin/loader" };
 		iput(bprm.inode);
 		loader = 1;
@@ -755,6 +766,7 @@ restart_interp:
 		if (retval)
 			goto exec_error1;
 		goto restart_interp;
+	    }
 	}
 #endif
 	if (!sh_bang) {
@@ -814,6 +826,7 @@ static int load_aout_binary(struct linux_binprm * bprm, struct pt_regs * regs)
 	unsigned long error;
 	unsigned long p = bprm->p;
 	unsigned long fd_offset;
+	unsigned long rlim;
 
 	ex = *((struct exec *) bprm->buf);		/* exec-header */
 	if ((N_MAGIC(ex) != ZMAGIC && N_MAGIC(ex) != OMAGIC && 
@@ -839,6 +852,16 @@ static int load_aout_binary(struct linux_binprm * bprm, struct pt_regs * regs)
 	}
 #endif
 
+	/* Check initial limits. This avoids letting people circumvent
+	 * size limits imposed on them by creating programs with large
+	 * arrays in the data or bss.
+	 */
+	rlim = current->rlim[RLIMIT_DATA].rlim_cur;
+	if (rlim >= RLIM_INFINITY)
+		rlim = ~0;
+	if (ex.a_data + ex.a_bss > rlim)
+		return -ENOMEM;
+
 	/* OK, This is the point of no return */
 	flush_old_exec(bprm);
 
