@@ -1,8 +1,6 @@
 #ifndef _LINUX_SCHED_H
 #define _LINUX_SCHED_H
 
-#include <linux/config.h>
-
 /*
  * define DEBUG if you want the wait-queues to have some extra
  * debugging code. It's not normally used, but might catch some
@@ -87,6 +85,13 @@ extern int nr_running, nr_tasks;
 #define TASK_STOPPED		4
 #define TASK_SWAPPING		5
 
+/*
+ * Scheduling policies
+ */
+#define SCHED_OTHER		0
+#define SCHED_FIFO		1
+#define SCHED_RR		2
+
 #ifndef NULL
 #define NULL ((void *) 0)
 #endif
@@ -128,10 +133,12 @@ struct fs_struct {
 struct mm_struct {
 	int count;
 	pgd_t * pgd;
+	unsigned long context;
 	unsigned long start_code, end_code, start_data, end_data;
 	unsigned long start_brk, brk, start_stack, start_mmap;
 	unsigned long arg_start, arg_end, env_start, env_end;
-	unsigned long rss;
+	unsigned long rss, total_vm, locked_vm;
+	unsigned long def_flags;
 	struct vm_area_struct * mmap;
 	struct vm_area_struct * mmap_avl;
 };
@@ -139,9 +146,11 @@ struct mm_struct {
 #define INIT_MM { \
 		1, \
 		swapper_pg_dir, \
+		0, \
 		0, 0, 0, 0, \
 		0, 0, 0, 0, \
 		0, 0, 0, 0, \
+		0, 0, 0, \
 		0, \
 		&init_mmap, &init_mmap }
 
@@ -186,7 +195,7 @@ struct task_struct {
 	struct wait_queue *wait_chldexit;	/* for wait4() */
 	unsigned short uid,euid,suid,fsuid;
 	unsigned short gid,egid,sgid,fsgid;
-	unsigned long timeout;
+	unsigned long timeout, policy;
 	unsigned long it_real_value, it_prof_value, it_virt_value;
 	unsigned long it_real_incr, it_prof_incr, it_virt_incr;
 	struct timer_list real_timer;
@@ -220,9 +229,10 @@ struct task_struct {
 	struct mm_struct *mm;
 /* signal handlers */
 	struct signal_struct *sig;
-#ifdef CONFIG_SMP
+#ifdef __SMP__
 	int processor;
-	int lock_depth;		/* Lock depth. We can context swithc in and out of holding a syscall kernel lock... */	
+	int last_processor;
+	int lock_depth;		/* Lock depth. We can context switch in and out of holding a syscall kernel lock... */	
 #endif	
 };
 
@@ -245,12 +255,14 @@ struct task_struct {
  */
 #define _STK_LIM	(8*1024*1024)
 
+#define DEF_PRIORITY	(20*HZ/100)	/* 200 ms time slices */
+
 /*
  *  INIT_TASK is used to set up the first task table, touch at
  * your own risk!. Base=0, limit=0x1fffff (=2MB)
  */
 #define INIT_TASK \
-/* state etc */	{ 0,15*HZ/100,15*HZ/100,0,0,0,0, \
+/* state etc */	{ 0,DEF_PRIORITY,DEF_PRIORITY,0,0,0,0, \
 /* debugregs */ { 0, },            \
 /* exec domain */&default_exec_domain, \
 /* binfmt */	NULL, \
@@ -261,15 +273,12 @@ struct task_struct {
 /* suppl grps*/ {NOGROUP,}, \
 /* proc links*/ &init_task,&init_task,NULL,NULL,NULL,NULL, \
 /* uid etc */	0,0,0,0,0,0,0,0, \
-/* timeout */	0,0,0,0,0,0,0, \
+/* timeout */	0,SCHED_OTHER,0,0,0,0,0,0, \
 /* timer */	{ NULL, NULL, 0, 0, it_real_fn }, \
 /* utime */	0,0,0,0,0, \
 /* flt */	0,0,0,0,0,0, \
 /* swp */	0,0,0,0,0, \
-/* rlimits */   { {LONG_MAX, LONG_MAX}, {LONG_MAX, LONG_MAX},  \
-		  {LONG_MAX, LONG_MAX}, {_STK_LIM, _STK_LIM},  \
-		  {       0, LONG_MAX}, {LONG_MAX, LONG_MAX}, \
-		  {MAX_TASKS_PER_USER, MAX_TASKS_PER_USER}, {NR_OPEN, NR_OPEN}}, \
+/* rlimits */   INIT_RLIMITS, \
 /* math */	0, \
 /* comm */	"swapper", \
 /* fs info */	0,NULL, \
@@ -291,7 +300,7 @@ extern struct task_struct *current_set[NR_CPUS];
  *	On a single processor system this comes out as current_set[0] when cpp
  *	has finished with it, which gcc will optimise away.
  */
-#define current (current_set[smp_processor_id()])	/* Current on this processor */
+#define current (0+current_set[smp_processor_id()])	/* Current on this processor */
 extern unsigned long volatile jiffies;
 extern unsigned long itimer_ticks;
 extern unsigned long itimer_next;
